"""Standard Django model definitions"""
from django.db import models
# Create your models here.
from django.db.models import BooleanField, CheckConstraint, Q


class MediumType(models.Model):
    """Enum for media types"""
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=255)
    description = models.CharField(max_length=255)

    def __str__(self):
        return self.description

    class Meta:
        """Definition of MySQL Table"""
        db_table = "mediumType"


class Medium(models.Model):
    """Model for media classes"""
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=255)
    medium_type = models.ForeignKey(MediumType, on_delete=models.CASCADE)

    def __str__(self):
        return self.name

    class Meta:
        """Definition of MySQL Table"""
        db_table = "medium"


class Tag(models.Model):
    """Model for created topic labels for articles"""
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=255)
    description = models.CharField(max_length=255)

    def __str__(self):
        return self.name

    class Meta:
        """Definition of MySQL Table"""
        db_table = "tag"


class Files(models.Model):
    """ model for uploaded file """
    upload_file = models.CharField(max_length=2055, default='no_file.pdf')
    upload_date = models.DateTimeField(auto_now_add=True)
    id = models.AutoField(primary_key=True)
    extraction_complete = models.BooleanField(default=False)

    class Meta:
        constraints = [
            CheckConstraint(check=Q(upload_file__contains='pdf'), name='articles_file_name')
        ]


class Article(models.Model):
    """Model of a single article, which is connected to a medium an several tags"""
    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=255)
    medium = models.ForeignKey(Medium, on_delete=models.CASCADE)
    source_file = models.ForeignKey(Files, on_delete=models.CASCADE, default=-1)
    date_of_publication = models.DateField('date published')
    reach = models.IntegerField(default=0)
    pages = models.IntegerField(default=0)
    text = models.TextField(default='')
    wrongly_parsed = BooleanField(default=False)
    tags = models.ManyToManyField(Tag, through='ArticleTag', through_fields=('article', 'tag',))

    def __str__(self):
        return self.name

    class Meta:
        """Definition of MySQL Table"""
        db_table = "article"


class ArticleTag(models.Model):
    """Model for many-to-many relation between article and tag"""
    article = models.ForeignKey(Article, on_delete=models.CASCADE)
    tag = models.ForeignKey(Tag, on_delete=models.CASCADE)

    class Meta:
        """Definition of MySQL Table"""
        db_table = "articleTag"


class Timeline(models.Model):
    """Model for accumulated article data (is a MySQL View) summarized by year and month"""
    name = models.CharField(max_length=255)
    year = models.IntegerField(default=1990)
    month = models.IntegerField(default=1)
    article_count = models.IntegerField(default=0)
    reach = models.DecimalField(max_digits=200, decimal_places=0, default=0, primary_key=True)

    def __str__(self):
        return self.name + ", " + \
               str(self.year) + ", " + \
               str(self.month) + ", " + \
               str(self.article_count) + ", " + \
               str(self.reach)

    class Meta:
        """Definition of MySQL Table"""
        managed = False
        db_table = "tag_reach"
        unique_together = (('reach', 'year'),)


class Overview(models.Model):
    """Model for compounded objects"""
    name = models.CharField(max_length=255)
    article_name = models.CharField(max_length=255)
    medium_name = models.CharField(max_length=255)
    date_of_publication = models.DateField('date published')
    reach = models.DecimalField(max_digits=200, decimal_places=0, default=0)

    class Meta:
        """Definition of MySQL Table"""
        managed = False
        db_table = "tag_article"


class TopMedia(models.Model):
    """ Model for top of media """
    name = models.CharField(max_length=255, primary_key=True)
    count = models.IntegerField(default=0)

    class Meta:
        """Definition of MySQL Table"""
        managed = False
        db_table = "top_media"


class TopTags(models.Model):
    """ Model for top of media """
    name = models.CharField(max_length=255, primary_key=True)
    count = models.IntegerField(default=0)

    class Meta:
        """Definition of MySQL Table"""
        managed = False
        db_table = "top_tags"


class User(models.Model):
    """ Model to configure User needs """
    id = models.AutoField(primary_key=True)
    id_tag = models.CharField(max_length=16)
    favorite_tags = models.ManyToManyField(Tag)
    activated = models.BooleanField(default=True)

    def __str__(self):
        return self.id_tag
