"""Standard django REST Framework serializer module"""

import logging

from rest_framework import serializers

from articles.models import Article, Medium, Timeline, MediumType, Overview, Tag, Files

# Get an instance of a logger
logger = logging.getLogger(__name__)


class MediumTypeSerializer(serializers.ModelSerializer):
    """ Serializer fpr MediumType """

    class Meta:
        """ Definition of model and used fields """
        model = MediumType
        fields = '__all__'

    def create(self, validated_data):
        """ Creates or updates a given mediumType """
        medium_type, created = MediumType.objects.update_or_create(name=validated_data.get('name', None),
                                                                   defaults={
                                                                       'description': validated_data.get('description',
                                                                                                         None)})
        if created:
            logger.debug("created new medium_type: %s", validated_data.get('name', None))

        return medium_type


class MediumSerializer(serializers.ModelSerializer):
    """ Serializer for media """

    class Meta:
        """ Definition of model and used fields """
        model = Medium
        fields = '__all__'

    # if the name of the print is already in database it just updates the medium type
    def create(self, validated_data):
        """ creates or updates a given medium with its medium_type """
        medium_type_str = validated_data.get('medium_type', None)
        medium_type, _ = MediumType.objects.update_or_create(name=medium_type_str,
                                                             defaults={
                                                                 'description': validated_data.get(
                                                                     'medium_type_description', '')})
        medium, created = Medium.objects.update_or_create(name=validated_data.get('name', None),
                                                          defaults={
                                                              'medium_type': medium_type})
        if created:
            logger.debug("created new medium: %s", validated_data.get('name', None))

        return medium


class ArticleSerializer(serializers.ModelSerializer):
    """ Serializer for articles """

    class Meta:
        """ Definition of model and used fields """
        model = Article
        fields = '__all__'

    # create or update
    def create(self, validated_data):
        """ creates or updates a new article with its medium and tags """
        # cut out medium data because we have to compute it differently
        medium_data = validated_data.pop('medium')
        if isinstance(medium_data, Medium):
            medium_obj = medium_data
        else:
            # I don't know why I init a new serializer but the one from the top is not accessible
            medium_ser = MediumSerializer(many=False)
            medium_obj = medium_ser.create(medium_data)
        article, created = Article.objects.update_or_create(medium_id=medium_obj.id,
                                                            source_file_id=validated_data.pop('source_file_id', -1),
                                                            **validated_data)

        if created:
            logger.debug("created new article: %s", validated_data.get('name', None))
        return article


class TimelineSerializer(serializers.ModelSerializer):
    """ Serializer for timeline """

    class Meta:
        """ Definition of model and used fields """
        model = Timeline
        fields = ['name', 'year', 'month', 'article_count', 'reach']


class OverviewSerializer(serializers.ModelSerializer):
    """ Serializer for overview """

    class Meta:
        """ Definition of model and used fields """
        model = Overview
        fields = ['name', 'article_name', 'medium_name', 'date_of_publication', 'reach']


def write_article_to_db(**kwargs):
    """ writes an article to the articles db
        :Keyword Arguments:
            * *medium_type* (``string``)
            defines the type of the medium in which the article was published
            * *medium_name* (``string``)
            the name of the medium in which the article was published
            * *source_file* (``string``)
            the file in which all articles are collected
            * *name* (``string``)
            title of the article
            * *date_of_publication* (``string``)
            Format: YYYY-MM-DD
            * *reach* (``int``)
            count of publications
            * *pages* (``int``)
            count of pages of the article
            * *text* (``string``)
            extracted text of the article
            * *wrongly_parsed* (``boolean``)
            True if there were problems in the process of information extraction
            * *tags* (``list``)
            Topic labels
    """
    medium_type_str = kwargs.pop('medium_type', 'no type')
    medium_name_str = kwargs.pop('medium_name', 'no name').lower()
    # I think it cannot be a case that we won't find a source_file but if this happens it causes damage
    source_file = kwargs.pop('source_file', None)
    logger.debug("write to db - medium_type: %s, medium_name: %s, source_file: %s", medium_type_str, medium_name_str,
                 source_file.upload_file)

    reach_ = kwargs.pop('reach', 0)
    if isinstance(reach_, str):
        # if reach_ is an empty string
        try:
            reach_ = int(reach_.replace('.', ''))
        except ValueError:
            reach_ = 0

    name_ = kwargs.pop('name', kwargs.pop('title', ''))

    tags = kwargs.pop('tags', [])

    medium_type, mt_created = MediumType.objects.update_or_create(name=medium_type_str)
    if mt_created:
        logger.debug("created new medium_type: %s", medium_type_str)

    medium_name_str = union_medium_with_submedia(medium_name_str)
    medium, medium_created = Medium.objects.update_or_create(name=medium_name_str,
                                                             medium_type=medium_type)
    if medium_created:
        logger.debug("created new medium: %s", medium_name_str)
    else:
        logger.debug("found medium: %s", medium_name_str)

    logger.debug("source file: %s", source_file.upload_file)
    article, article_created = Article.objects.update_or_create(medium=medium, source_file=source_file, name=name_,
                                                                reach=reach_, **kwargs)

    if article_created:
        logger.debug("created new article: %s", name_)

    add_tags_to_article(article, tags)
    article.save()


# pylint: disable=too-many-branches
def union_medium_with_submedia(medium_name_str):
    """ union sub media with main medium, like 'Märkische Allgemeine' (main medium) and 'Märkische Allgemeine -
    Westhavelländer', 'Märkische Allgemeine - Potsdamer Tageszeitung' """
    if Medium.objects.filter(name=medium_name_str).count() == 0:
        if 'lausitzer rundschau' in medium_name_str:
            medium_name_str = 'lausitzer rundschau'
        elif 'welt am sonntag' in medium_name_str:
            medium_name_str = 'welt am sonntag'
        elif 'welt.de' in medium_name_str or 'welt online' in medium_name_str:  # welt.de und welt online
            medium_name_str = 'welt online'
        elif 'welt' in medium_name_str and 'junge' not in medium_name_str:
            medium_name_str = 'die welt'
        elif medium_name_str == 'dw':
            medium_name_str = 'die welt'
        elif medium_name_str in ('nd', 'nd (neues deutschland)'):
            medium_name_str = 'neues deutschland'
        elif 'wochenspiegel' in medium_name_str:
            medium_name_str = 'wochenspiegel'
        elif medium_name_str == 'bildplus':
            medium_name_str = 'bild'
        elif 'bild' in medium_name_str and 'frau' not in medium_name_str:
            medium_name_str = 'bild'
        elif '+' in medium_name_str:
            medium_name_str = medium_name_str.split('+')[0]
            if medium_name_str == 'spiegel':
                medium_name_str = 'der spiegel'
        elif 'nordkurier' in medium_name_str:
            medium_name_str = 'nordkurier'
        elif 'schweriner volkszeitung' in medium_name_str:
            medium_name_str = 'schweriner volkszeitung'
        elif 'frankfurter rundschau' in medium_name_str:
            medium_name_str = 'frankfurter rundschau'
        elif 'nordwest-zeitung' in medium_name_str:
            medium_name_str = 'nordwest-zeitung'
        else:
            medium_splitted_name = medium_name_str.split(' - ')[0]  # Märkische Zeitung und Märkische Oderzeitung
            if Medium.objects.filter(name=medium_splitted_name).count() == 1:
                medium_name_str = medium_splitted_name

    return medium_name_str.rstrip()


# pylint: enable=too-many-branches

def add_tags_to_article(article, tags):
    """ The tags need to be added one by one because of the many to many relation between tags and articles """
    for tag_name in tags:
        tag, tag_created = Tag.objects.update_or_create(name=tag_name)
        if tag_created:
            logger.debug("created new tag: %s", tag_name)
        article.tags.add(tag)


class FileSerializer(serializers.ModelSerializer):
    """ Serializer for files """

    class Meta:
        """ Definition of model and used fields """
        model = Files
        fields = ['name']


class TopMediaSerializer(serializers.Serializer):
    """ Serializer class to shows how many articles are in each medium """
    medium_name = serializers.CharField(max_length=200)
    article_count = serializers.IntegerField()

    def update(self, instance, validated_data):
        """ not yet needed """

    def create(self, validated_data):
        """ not yet needed """


class TopTagsSerializer(serializers.Serializer):
    """ Serializer class to shows how many articles are in each medium """
    tag_name = serializers.CharField(max_length=200)
    article_count = serializers.IntegerField()
    hotness_factor = serializers.FloatField()

    def update(self, instance, validated_data):
        """ not yet needed """

    def create(self, validated_data):
        """ not yet needed """


class TagSerializer(serializers.ModelSerializer):
    """ Serializer for tags"""

    class Meta:
        """ Definition of model and used fields """
        model = Tag
        fields = ['name']
