""" Test class for article module """
import datetime
import logging
from unittest import mock

from django.db.models.signals import post_save
from django.test import TransactionTestCase

from articles.apps import ArticlesConfig
from articles.models import Article, MediumType, Medium, Files, User, Tag
from articles.serializers import MediumTypeSerializer, MediumSerializer, ArticleSerializer, write_article_to_db

logger = logging.getLogger(__name__)


class MediumTypeTest(TransactionTestCase):
    """ database test medium_type """
    reset_sequences = True

    def setUp(self):
        logger.info("started MediumTypeTest.setup")
        MediumType.objects.create(name='local', description='Lokale Zeitung')

    def test_db_call(self):
        """ Tests get medium_type db call """
        logger.info("started MediumTypeTest.test_db_call")
        local_print = MediumType.objects.get(name='local')
        self.assertEqual(local_print.description, 'Lokale Zeitung')
        self.assertEqual(local_print.id, 1)

    def test_serializer(self):
        """ Tests get medium_type serializer"""
        logger.info("started MediumTypeTest.test_serializer")
        MediumTypeSerializer().create({'name': 'regional', 'description': 'Regionale Zeitung'})
        self.assertEqual(MediumType.objects.count(), 2)


class MediumTest(TransactionTestCase):
    """ database test medium """
    reset_sequences = True

    def setUp(self):
        logger.info("started MediumTest.setup")
        MediumType.objects.create(name='new local', description='Lokale Zeitung')
        Medium.objects.create(name='Meine Zeitung', medium_type_id=1)

    def test_db_call(self):
        """ Tests get media """
        logger.info("started MediumTest.test_db_call")
        my_print = Medium.objects.get(name='Meine Zeitung')
        self.assertEqual(my_print.name, 'Meine Zeitung')
        self.assertEqual(my_print.medium_type.name, 'new local')
        self.assertEqual(my_print.medium_type.id, 1)

    def test_serializer(self):
        """ Tests get Medium serializer"""
        logger.info("started MediumTest.test_serializer")
        MediumSerializer().create({'medium_type': 'new local', 'name': 'Neue Zeitung'})
        self.assertEqual(MediumType.objects.count(), 1)
        self.assertEqual(Medium.objects.count(), 2)


class ArticleTest(TransactionTestCase):
    """ database test medium """
    reset_sequences = True

    def setUp(self):
        logger.info("started ArticleTest.setup")
        with mock.patch('articles.signals.extract_information_from_file', autospec=True) as mocked_post_save:
            post_save.connect(mocked_post_save, sender=Files, dispatch_uid='extract_information_from_file')
            MediumType.objects.create(name='my local', description='Lokale Zeitung')
            Medium.objects.create(name='JDs Zeitung', medium_type_id=1)
            Files.objects.create(upload_file='testfile.pdf', upload_date='18.11.2021')
            input_data = {
                'name': 'Testartikel',
                'date_of_publication': '1988-12-30',
                'reach': 1000,
                'pages': 2,
                'text': 'Dies ist ein beispielhafter Beispieltext',
                'wrongly_parsed': True,
            }
            Article.objects.create(medium_id=1, source_file_id=1, **input_data)

    def test_author(self):
        """ Tests get medium_type """
        logger.info("started test_author")
        article = Article.objects.get(name='Testartikel')
        self.assertEqual(article.date_of_publication, datetime.date(1988, 12, 30))
        self.assertEqual(article.medium.name, 'JDs Zeitung')

    def test_serializer(self):
        """ Tests the article serializer """
        logger.info("started ArticleTest.test_serializer")
        input_data = {
            'name': 'Testartikel2',
            'date_of_publication': '2021-12-30',
            'reach': 2000,
            'pages': 3,
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'wrongly_parsed': True,
            'source_file_id': 1,
        }
        medium = Medium.objects.get(name='JDs Zeitung')
        input_data_ = {'medium': medium, **input_data}
        ArticleSerializer().create(input_data_)
        self.assertEqual(Article.objects.count(), 2)


def create_articles_with_tags():
    """ prepare test data """
    tag_1 = Tag(name="Test1")
    tag_1.save()
    tag_2 = Tag(name="Test2")
    tag_2.save()
    conf = ArticlesConfig.create('articles.apps.ArticlesConfig')
    conf.ready()
    with mock.patch('articles.signals.extract_information_from_file', autospec=True) as mocked_post_save:
        post_save.connect(mocked_post_save, sender=Files, dispatch_uid='extract_information_from_file')
        Files.objects.create(upload_file='testfile.pdf', upload_date='18.11.2021')
        file = Files.objects.get(upload_file='testfile.pdf')
        input_data = {
            'medium_name': 'MAZ',
            'medium_type': 'Zeitung',
            'name': 'Testartikel',
            'source_file': file,
            'reach': 1000,
            'date_of_publication': datetime.datetime.today(),
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': True,
            'tags': [tag_1.name, tag_2.name]
        }
        write_article_to_db(**input_data)


class OverviewTest(TransactionTestCase):
    """ test functions for REST API Overview"""

    def setUp(self):
        create_articles_with_tags()

    def test_create_user_app_conf(self):
        """ tests if user were created if config is ready"""
        logger.info("started OverviewTest.test_create_user_app_conf")
        user = User.objects.get(id_tag=1)
        self.assertTrue(user.activated)
        self.assertEqual(user.favorite_tags.all().count(), 2)

    def test_overview_api_wo_user(self):
        """ tests the api without user id """
        logger.info("started OverviewTest.test_overview_api_wo_user")
        response = self.client.get('/articles/overview')
        self.assertEqual(response.status_code, 200)
        today_str = datetime.datetime.today().strftime('%Y-%m-%d')
        self.assertJSONEqual(str(response.content, encoding='utf8'), [
            {"name": "Test1", "article_name": "Testartikel", "medium_name": "maz",
             "date_of_publication": today_str,
             "reach": "1000"
             },
            {"name": "Test2", "article_name": "Testartikel", "medium_name": "maz",
             "date_of_publication": today_str,
             "reach": "1000"}])

    def test_overview_api_with_user(self):
        """ tests the overview with user id """
        logger.info("started OverviewTest.test_overview_api_with_user")
        today_str = datetime.datetime.today().strftime('%Y-%m-%d')
        user = User(id_tag=2)
        user.save()
        tag2 = Tag.objects.get(name="Test2")
        user.favorite_tags.add(tag2)
        user.save()
        response = self.client.get('/articles/overview?user=' + str(user.id_tag))
        self.assertJSONEqual(str(response.content, encoding='utf8'), [
            {"name": tag2.name, "article_name": "Testartikel", "medium_name": "maz",
             "date_of_publication": today_str,
             "reach": "1000"}
        ])


class TopMediaTest(TransactionTestCase):
    """ Tests the top media api"""

    def setUp(self):
        create_articles_with_tags()

    def test_top_media_blank_api_wo_user(self):
        """ tests if the link works and returns the exact article count """
        response = self.client.get('/articles/top/media')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'article_count': 1, 'medium_name': 'maz'}])

    def test_top_media_with_extra_articles(self):
        """ test the setting with multiple media """
        file = Files.objects.get(upload_file='testfile.pdf')
        tag_1 = Tag.objects.get(name="Test1")
        tag_2 = Tag.objects.get(name="Test2")
        input_data = {
            'medium_name': 'MOZ',
            'medium_type': 'Zeitung',
            'name': 'Testartikel2',
            'source_file': file,
            'reach': 1111,
            'date_of_publication': datetime.datetime.today(),
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': True,
            'tags': [tag_1.name]
        }
        write_article_to_db(**input_data)
        input_data['name'] = 'Testartikel3'
        input_data['tags'] = [tag_2.name]
        write_article_to_db(**input_data)
        response = self.client.get('/articles/top/media')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{'article_count': 2, 'medium_name': 'moz'}, {'article_count': 1, 'medium_name': 'maz'}])

    def test_top_media_with_extra_tag(self):
        """ tests how the api is dealing with tags """
        file = Files.objects.get(upload_file='testfile.pdf')
        tag3_name = "Test3"
        input_data = {
            'medium_name': 'MOZ',
            'medium_type': 'Zeitung',
            'name': 'Testartikel2',
            'source_file': file,
            'reach': 1111,
            'date_of_publication': datetime.datetime.today(),
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': True,
            'tags': [tag3_name]
        }
        write_article_to_db(**input_data)
        response = self.client.get('/articles/top/media')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'article_count': 1, 'medium_name': 'maz'}])
        user = User.objects.get(id_tag=1)
        tag3 = Tag.objects.get(name=tag3_name)
        user.favorite_tags.add(tag3)
        response = self.client.get('/articles/top/media')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{'article_count': 1, 'medium_name': 'maz'}, {'article_count': 1, 'medium_name': 'moz'}])


class HotTopicTest(TransactionTestCase):
    """ Tests the hot topic api """

    def setUp(self):
        create_articles_with_tags()

    def test_top_tag_blank(self):
        """ tests if the link works and returns the exact article count """
        response = self.client.get('/articles/top/tags')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'),
                             [{'article_count': 1, 'hotness_factor': 0.5, 'tag_name': 'Test1'},
                              {'article_count': 1, 'hotness_factor': 0.5, 'tag_name': 'Test2'}])


class UserTagsTest(TransactionTestCase):
    """ Tests how to get and set tags for users """

    def setUp(self):
        create_articles_with_tags()

    def test_get_standard_user_tags(self):
        """ Tests with standard user"""
        response = self.client.get('/articles/user/tags')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Test1'}, {'name': 'Test2'}])

    def test_get_new_user_tags(self):
        """ creates another user and tests his tags """
        user = User(id_tag=14482)
        user.save()
        response = self.client.get('/articles/user/tags?user=14482')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [])
        tag = Tag(name="Test3")
        tag.save()
        user.favorite_tags.add(tag)
        response = self.client.get('/articles/user/tags?user=14482')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Test3'}])

    def test_post_tags(self):
        """ test post tags """
        response = self.client.get('/articles/user/tags')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Test1'}, {'name': 'Test2'}])
        response = self.client.post('/articles/user/tags', {'user': 1, 'tags': ['Test1']})
        self.assertEqual(response.status_code, 201)
        response = self.client.get('/articles/user/tags')
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(str(response.content, encoding='utf8'), [{'name': 'Test1'}])

    def test_post_no_user(self):
        """ tests api with wrong post attribute (user) """
        response = self.client.post('/articles/user/tags', {'tags': ['Test1']})
        self.assertEqual(response.status_code, 400)

    def test_post_no_tags(self):
        """ tests api with wrong post attribute (tags) """
        response = self.client.post('/articles/user/tags', {'user': 1})
        self.assertEqual(response.status_code, 400)

    def test_post_no_correct_user(self):
        """ tests api with wrong post attribute (user id) """
        response = self.client.post('/articles/user/tags', {'user': 1111, 'tags': ['Test1']})
        self.assertEqual(response.status_code, 400)
