""" Views with OpenAPI annotations """
import datetime
import logging

from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q, Count, Sum
from django.http import JsonResponse
from django.shortcuts import render
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.views import View
from django.views.generic import TemplateView
from drf_yasg import openapi
from drf_yasg.utils import swagger_auto_schema
from rest_framework import generics
from rest_framework.response import Response
from rest_framework.views import APIView

from articles.forms import FileUploadForm
from articles.models import Article, Timeline, Medium, MediumType, Overview, TopMedia, Tag, TopTags, Files, User
from articles.serializers import ArticleSerializer, TimelineSerializer, MediumSerializer, MediumTypeSerializer, \
    OverviewSerializer, TopMediaSerializer, TagSerializer, TopTagsSerializer
from articles.tasks import read_all_media_files

logger = logging.getLogger(__name__)


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="gets a list of all articles"
))
@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="inserts an new article or updates if the name is already in the db"
))
class ArticleList(generics.ListCreateAPIView):
    """ Lists all articles """
    queryset = Article.objects.all()
    serializer_class = ArticleSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows all details about one article selected by Id"
))
class ArticleDetail(generics.RetrieveUpdateDestroyAPIView):  # pylint: disable=too-many-ancestors
    """ Get details of article """
    queryset = Article.objects.all()
    serializer_class = ArticleSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="gets a list of all medias"
))
@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="inserts an new medium or updates if the name is already in the db"
))
class MediumList(generics.ListCreateAPIView):
    """ List all media """
    queryset = Medium.objects.all()
    serializer_class = MediumSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows all details about one medium selected by Id"
))
class MediumDetail(generics.RetrieveUpdateDestroyAPIView):  # pylint: disable=too-many-ancestors
    """ Gets details of medium """
    queryset = Medium.objects.all()
    serializer_class = MediumSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="gets a list of all types of media"
))
@method_decorator(name='post', decorator=swagger_auto_schema(
    operation_description="inserts an new medium_type or updates if the name is already in the db"
))
class MediumTypeList(generics.ListCreateAPIView):
    """ List all medium_types """
    queryset = MediumType.objects.all()
    serializer_class = MediumTypeSerializer


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows all details about one medium_type selected by Id"
))
class MediumTypeDetail(generics.RetrieveUpdateDestroyAPIView):  # pylint: disable=too-many-ancestors
    """ Get details of medium_type """
    queryset = MediumType.objects.all()
    serializer_class = MediumTypeSerializer


def user_tags(request_params):
    """ get favorite tags by generated user id """
    id_tag = 1
    if request_params.__contains__('user'):
        id_tag = request_params['user']
    try:
        user = User.objects.get(id_tag=id_tag)
    except ObjectDoesNotExist:
        logger.error('User with id_tag %s does not exists', str(id_tag))
        return []
    tag_names = []
    for tag in user.favorite_tags.all():
        tag_names.append(tag.name)
    return tag_names


@method_decorator(name='get', decorator=swagger_auto_schema(
    operation_description="shows how the tags perform over time"
))
class TimelineList(generics.ListAPIView):
    """ List all timeline objects """
    today = datetime.datetime.today()
    datetime_recent = today - datetime.timedelta(days=365)
    queryset = Timeline.objects.filter(
        Q(year=today.year, month__lte=today.month) | Q(year=datetime_recent.year, month__gte=datetime_recent.month))
    serializer_class = TimelineSerializer


user_param_get = openapi.Parameter('user', openapi.IN_QUERY, description="id which is connected to the user",
                                   type=openapi.TYPE_NUMBER)


class OverviewList(APIView):
    """ List all overview objects """
    today = datetime.datetime.today()
    datetime_recent = today - datetime.timedelta(days=90)

    @swagger_auto_schema(manual_parameters=[user_param_get], responses={200: OverviewSerializer(many=True)})
    def get(self, request):
        """ get overview, a user can be selected with user=user_id """
        tag_names = user_tags(request.query_params)
        overview_objects = Overview.objects.filter(
            date_of_publication__range=[self.datetime_recent, self.today]).filter(name__in=tag_names)

        overview_serializer = OverviewSerializer(overview_objects, many=True)
        return JsonResponse(overview_serializer.data, safe=False)


class Home(TemplateView):
    """ Landing page """
    template_name = 'wizard/home.html'

    def get_context_data(self, **kwargs):
        """ generate context data for Landing Page"""
        context = super().get_context_data(**kwargs)
        context['site_urls'] = {
            'articles': [
                {'name': 'open-api', 'url': '/swagger/'},
                {'name': 'articles', 'url': '/articles/articles'},
                {'name': 'media', 'url': '/articles/media'},
                {'name': 'medium type', 'url': '/articles/mediumtype'},
                {'name': 'by Tags', 'url': '/articles/byTags'},
                {'name': 'overview', 'url': '/articles/overview'},
            ],
            'pr_upload': [
                {'name': 'upload a file', 'url': '/pr_upload'},
            ],
            'wizard': [
                {'name': 'Wizard Overview', 'url': '/wizard/overview'},
            ],
            'projekt': [
                {'name': 'Repository', 'url': 'https://gitup.uni-potsdam.de/innohs/rest-api'},
                {'name': 'Projektbeschreibung',
                 'url': 'https://www.uni-potsdam.de/de/multimedia/projekte/institutionelle-infrastrukturen/innovative-hochschule'},
            ],
        }
        return context


class OverviewDashboard(TemplateView):
    """ Wizard pages """
    template_name = 'wizard/overview.html'

    def get_context_data(self, **kwargs):
        """ generate context data for overview Page"""
        article_count = Article.objects.count()
        tag_count = Tag.objects.count()

        context = super().get_context_data(**kwargs)
        context['articles'] = {
            'count': article_count
        }
        context['media'] = {
            'count': Medium.objects.count(),
            'top': TopMedia.objects.all()
        }
        context['tags'] = {
            'count': tag_count,
            'top': TopTags.objects.all()
        }
        context['files'] = {
            'count': Files.objects.count()
        }
        context['users'] = {
            'count': User.objects.count()
        }
        if (article_count == 0) or (tag_count == 0):
            context['ratio'] = {'apt': 0, 'tpa': 0}
        else:
            context['ratio'] = {
                'apt': article_count / tag_count,
                'tpa': tag_count / article_count
            }
        return context


class Simulator(View):
    """ Wizard pages """
    template_name = 'wizard/simulator.html'
    selected_user = {}

    def get(self, request, *args, **kwargs):
        """ Answer to GET request """
        logger.debug("GET Simulator")
        return render(request, self.template_name, {})

    def post(self, request, *args, **kwargs):
        """ Answer to POST request """
        logger.info("POST Simulator")
        read_all_media_files.delay()
        return render(request, self.template_name, {})


class UploadView(View):
    """ single file upload view """
    template_name = 'articles/files_form.html'
    form_class = FileUploadForm

    def get(self, request, *args, **kwargs):
        """ get of UploadView """
        logger.debug("GET Uploadview")
        form = self.form_class(initial={'file_name': 'no_file.pdf'})
        return render(request, self.template_name, {'documents': Files.objects.all(), 'form': form})

    def post(self, request, *args, **kwargs):
        """ post of UploadView """
        logger.info("POST Uploadview")
        form = self.form_class(request.POST)
        if form.is_valid():
            name_ = form.cleaned_data['file_name']

            logger.debug("debug %i", Files.objects.filter(upload_file=name_).count())
            if Files.objects.filter(upload_file=name_).count() == 0:
                file = Files(upload_file=name_, upload_date=timezone.now())
                file.save()
        return render(request, self.template_name, {'documents': Files.objects.all(), 'form': form})


class TopMediaView(APIView):
    """ gets for a specific user and his favorite tags the top media platforms """

    @swagger_auto_schema(manual_parameters=[user_param_get], responses={200: TopMediaSerializer(many=True)})
    def get(self, request):
        """ get method """
        tag_names = user_tags(request.query_params)
        # Counts the number of articles in a medium depending on the tags
        top_media = Medium.objects.filter(article__tags__name__in=tag_names).annotate(
            article_count=Count('article', distinct=True)).order_by('-article_count')
        raw_media_data = []
        for medium in top_media.all():
            raw_media_data.append({'medium_name': medium.name, 'article_count': medium.article_count})
            if len(raw_media_data) > 5:
                break
        top_media_serializer = TopMediaSerializer(raw_media_data, many=True)
        return JsonResponse(top_media_serializer.data, safe=False)


def unwanted_request(request, message):
    """ if there are missing attributes in the request data this function is taking care of logging """
    logger.warning(message)
    logger.warning(request.data)
    return Response({'message': message}, 400)


user_param_post = openapi.Parameter('user', openapi.IN_BODY, description="id which is connected to the user",
                                    type=openapi.TYPE_NUMBER)
tags_param_post = openapi.Parameter('tags', openapi.IN_BODY, description="id which is connected to the user",
                                    type=openapi.TYPE_ARRAY, items=openapi.Items(type=openapi.TYPE_STRING))


class UserFavoriteTags(APIView):
    """ view to get and set favorite tags from user """

    @swagger_auto_schema(manual_parameters=[user_param_get], responses={200: TagSerializer(many=True)})
    def get(self, request):
        """ get tags for specific user """
        tag_names = user_tags(request.query_params)
        tags = Tag.objects.filter(name__in=tag_names)
        tag_serializer = TagSerializer(tags, many=True)
        return JsonResponse(tag_serializer.data, safe=False)

    # doesnt work yet
    @swagger_auto_schema(request_body=openapi.Schema(
        type=openapi.TYPE_OBJECT,
        properties={
            'user': openapi.Schema(type=openapi.TYPE_NUMBER, description='user_id'),
            'tags': openapi.Schema(type=openapi.TYPE_ARRAY, items=openapi.Items(type=openapi.TYPE_STRING),
                                   description='tags which should be saved'),
        }),
        responses={201: 'tags are set', 400: 'an parameter is wrongly set'})
    def post(self, request):
        """ safes all given tags to a specific user """
        if request.data.__contains__('user'):
            if request.data.__contains__('tags'):
                try:
                    user = User.objects.get(id_tag=request.data['user'])

                except ObjectDoesNotExist:
                    return unwanted_request(request, 'User not found')
                user.favorite_tags.remove(*user.favorite_tags.all())
                user.favorite_tags.add(*Tag.objects.filter(name__in=request.data['tags'].split(', ')).all())
                logger.info("Added tags.")
                logger.info(request.data)
                return Response({'message': 'Hello World!'}, 201)
            return unwanted_request(request, 'Couldn\'t update tags without tags')
        return unwanted_request(request, 'Couldn\'t update tags without user id')


class HotTopics(APIView):
    """ this class gives the hot hot hot topics!!! """

    @swagger_auto_schema(responses={200: TopTagsSerializer(many=True)})
    def get(self, request):
        """ get method """
        today = datetime.datetime.today()
        datetime_recent = today - datetime.timedelta(days=30)
        tags = Tag.objects.filter(article__date_of_publication__range=[datetime_recent, today]).annotate(
            article_count=Count('article', distinct=True)).order_by('-article_count')
        tag_sum = tags.aggregate(Sum('article_count'))
        top_tags = []
        for tag in tags.all():
            top_tags.append({'tag_name': tag.name, 'article_count': tag.article_count,
                             'hotness_factor': tag.article_count / tag_sum['article_count__sum']})
        top_tags_serializer = TopTagsSerializer(top_tags, many=True)
        return JsonResponse(top_tags_serializer.data, safe=False)
