#!/usr/local/bin/python
"""
Why implement a python based entrypoint?
 - subprocess waits for process completion and thus eliminates the need for a wait-loop
 - enables the use of smaller docker images omitting the shell requirement
"""
import logging
import os.path
import subprocess
import sys

PYTHON_INTERPRETER: str = "/usr/local/bin/python"
DJANGO_MANAGE: str = "manage.py"
logger = logging.getLogger(__name__)


def check_for_tools():
    """
    check if the used tools are there.

    :return: nothing
    """

    if not os.path.isfile(PYTHON_INTERPRETER):
        sys.exit("Interpreter " + PYTHON_INTERPRETER + " not found.")

    if not os.path.isfile(DJANGO_MANAGE):
        sys.exit(DJANGO_MANAGE + " file is missing.")


def run_django_manage(django_cmd, django_cmd_parameter=""):
    """
    Runs given django command with given parameter.

    :param django_cmd: command to execute with manage.py
    :param django_cmd_parameter: parameter for the fore mentioned command
    :return: Popen object
    """

    try:
        parameter_ = list(filter(None, [PYTHON_INTERPRETER,
                                        DJANGO_MANAGE,
                                        django_cmd,
                                        django_cmd_parameter]))
        return subprocess.call(parameter_)
    except FileNotFoundError as error:
        sys.exit(error.strerror)
    except subprocess.CalledProcessError as error:
        logger.debug(django_cmd + " exited with non zero exit code.")
        sys.exit(error.returncode)


logger.debug("check if later used tools are there")
check_for_tools()
logger.debug("make migrations")
run_django_manage("makemigrations")
logger.debug("Migrate the database at startup.")
run_django_manage("migrate", "--fake-initial")
# logger.debug("start celery")
# subprocess.Popen(["celery", "-A", "one_api", "worker", "-l", "INFO"])  # pylint: disable=R1732
logger.debug("Migration done. Starting Django")
run_django_manage("runserver", "0.0.0.0:8000")
